package game;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

/**
 * This Dealer class runs a game with a number of players and a user at the
 * keyboard. The object of the game is for the user at the keyboard to guess
 * numbers that the players will find acceptable. As each player gets a number
 * that it likes, it will leave the table (exit its run method). The purpose
 * of this class is to demonstrate some simple aspects of threading.
 */
public class Dealer implements Runnable {
  private BufferedReader consoleIn;
  private PrintStream consoleOut;

  /**
   * The board is a little data holder. The players and the dealer use it for
   * data transfer and synchronization purposes.
   */
  private GameBoard board;
  /**
   * Number of players (threads) that can run simultaneously
   */
  private int seatCount;
  /**
   * The gameBoss limits the number of player threads running
   */
  private ExecutorService gameBoss;
  /**
   * List of players.
   */
  private List<BasicPlayer> players;
  /**
   * Future<String>> results produced by the player threads.
   */
  private List<Future<String>> results;

  /**
   * Initialize this Dealer.
   * @param players number of players that will participate in the game
   * @param seats maximum number of active player threads allowed at any one
   *          time
   */
  public Dealer(int playerCount, int seats) {
    consoleIn = new BufferedReader(new InputStreamReader(System.in));
    // consoleOut = new PrintWriter(new OutputStreamWriter(System.out), true);
    consoleOut = System.out;
    board = new GameBoard();

    seatCount = seats;
    gameBoss = Executors.newFixedThreadPool(seatCount);

    players = new ArrayList<BasicPlayer>();
    results = new ArrayList<Future<String>>();

    BasicPlayer odd = new OddNumbers(board);
    BasicPlayer neg = new NegativeNumbers(board);

    for (int i = 0; i < playerCount; i++) {
      if (Math.random() < 0.5) {
        players.add(odd);
      } else {
        players.add(neg);
      }
    }
  }

  /**
   * Run the game by starting the players up and then asking the user for
   * numbers to supply to the players. Keep running until the user types a
   * blank line.
   */
  public void run() {
    
    board.setLatch(seatCount);
    for (BasicPlayer p : players) {
      results.add(gameBoss.submit(p));
    }

    // Get numbers from the user at the keyboard and
    // let the players take a look at them.
    // Keep track of the number of guesses it takes to guess numbers that
    // satisfy all of the players.

    int guessCount = 0;

    try {
      do {
        board.awaitLatch(750);  // wait for the players to get ready
        consoleOut.print(System.getProperty("line.separator"));
        consoleOut.print("Your integer guess? ");
        String line = consoleIn.readLine();

        if (line == null || line.length() == 0) {
          break;
        }
        int newGuess = Integer.parseInt(line);
        guessCount++;
        synchronized (board) {
          board.setGuess(newGuess);
          board.setLatch(seatCount);
          board.notifyAll();
        }
      } while (true);
    } catch (Exception e) {
      consoleOut.println(e);
    }
    consoleOut.println("Game over.");
    try {
      gameBoss.shutdownNow();
      gameBoss.awaitTermination(750, TimeUnit.MILLISECONDS);
      for (Future<String> r : results) {
        if (r.isDone()) {
          try {
            consoleOut.print("Status: Task is done.");
            consoleOut.println(" Result: " + r.get());
          } catch (Exception e) {
            consoleOut.println(e);
          }
        } else if (r.isCancelled()) {
          consoleOut.println("Status: Task was cancelled.");
        } else {
          consoleOut.println("Status: Task never ran.");
        }
      }
    } catch (InterruptedException e) {
      consoleOut.println(e);
    }
  }
}
